#!/usr/bin/python

# Copyright (c) 2008 Volker Mische (http://vmx.cx/)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import unittest
import pythonpath

class TestPythonPathParser(unittest.TestCase):

    def testDictDepthOne(self):
        """parser: testDictDepthOne"""
        path = 'person'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 1,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, ['person'],
                         'not correctly splitted (%s)' % items)

    def testDictDepthTwo(self):
        """parser: testDictDepthTwo"""
        path = 'person.name'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 2,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, ['person', 'name'],
                         'not correctly splitted (%s)' % items)

    def testDictDepthThree(self):
        """parser: testDictDepthThree"""
        path = 'person.name.surname'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 3, 'item count is wrong')
        self.assertEqual(items, ['person', 'name', 'surname'],
                         'not correctly splitted')


    def testListFirst(self):
        """parse: testListFirst"""
        path = '[0]'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 1,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, [0],
                         'not correctly splitted (%s)' % items)

    def testListSecond(self):
        """parse: testListSecond"""
        path = '[0][54]'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 2,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, [0, 54],
                         'not correctly splitted (%s)' % items)

    def testListThird(self):
        """parse: testListThird"""
        path = '[0][541][14]'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 3,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, [0, 541, 14],
                         'not correctly splitted (%s)' % items)

    def testMixedFirst(self):
        path = 'person[0].name.surname'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 4, 'item count is wrong')
        self.assertEqual(items, ['person', 0, 'name', 'surname'],
                         'not correctly splitted')

    def testMixedSecond(self):
        path = 'person.name[0].surname'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 4, 'item count is wrong')
        self.assertEqual(items, ['person', 'name', 0, 'surname'],
                         'not correctly splitted')

    def testMixedThird(self):
        path = 'person.name.surname[0]'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 4, 'item count is wrong')
        self.assertEqual(items, ['person', 'name', 'surname', 0],
                         'not correctly splitted')

    def testMixedFirstAndThird(self):
        path = 'person[0].name.surname[0]'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 5, 'item count is wrong')
        self.assertEqual(items, ['person', 0, 'name', 'surname', 0],
                         'not correctly splitted')

    def testMixedOnes(self):
        path = 'person[3]'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 2, 'item count is wrong')
        self.assertEqual(items, ['person', 3], 'not correctly splitted')

    def testMixedTens(self):
        path = 'person[34]'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 2, 'item count is wrong')
        self.assertEqual(items, ['person', 34], 'not correctly splitted')

    def testMixedHundreds(self):
        path = 'person[341]'
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 2, 'item count is wrong')
        self.assertEqual(items, ['person', 341], 'not correctly splitted')


    def testEscapingDotDepthOne(self):
        path = 'per\.son';
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 1,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, ['per.son'], 'not correctly splitted')

    def testEscapingDotDepthTwo(self):
        path = 'person.na\.me';
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 2,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, ['person', 'na.me'],
                         'not correctly splitted (%s)' % items)

    def testEscapingBracketDepthOne(self):
        path = 'person\[0\]';
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 1,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, ['person[0]'],
                         'not correctly splitted (%s)' % items)

    def testEscapingBracketDepthTwo(self):
        path = 'person[0].name\[0\]';
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 3,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, ['person', 0, 'name[0]'],
                         'not correctly splitted (%s)' % items)

    def testEscapingBackslash(self):
        path = 'per\\\\son';
        items = pythonpath.parser(path)
        self.assertEqual(len(items), 1,
                         'item count is wrong (%d)' % len(items))
        self.assertEqual(items, ['per\\son'],
                         'not correctly splitted (%s)' % items)



class TestPythonPathGetItem(unittest.TestCase):

    def setUp(self):
        self.datadict = {'person':
            {'name': {
                'forename': 'Alice',
                'surname': 'Abc'}
            }
        }

        self.datalist = [
            ['Alice', 'Abc'],
            ['Bob', 'Bcd'],
            ['Charly', 'Cdb']
        ]

        self.datamixed1 = {'person': [
            {'name': {
                'forename': 'Alice',
                'surname': 'Abc'}
            },
            {'name': {
                'fore.name': 'Bob',
                'surname': 'Bcd'}
            }
        ]}

        self.datamixed2 = {'person': 
            {'name': [
                {
                 'forename': 'Alice',
                 'surname': 'Abc'
                },
                {
                 'fore.name': 'Bob',
                 'surname': 'Bcd'
                }
            ]}
        }




    def testGetDictDepthOne(self):
        """get_item: testGetDictDepthOne"""
        path = 'person'
        item = pythonpath.get_item(path, self.datadict)
        self.assertEqual(item,
                         {'name': {'surname': 'Abc', 'forename': 'Alice'}},
                         'Wrong item was selected (%s)' % item)

    def testGetDictDepthTwo(self):
        """get_item: testGetDictDepthTwo"""
        path = 'person.name'
        item = pythonpath.get_item(path, self.datadict)
        self.assertEqual(item, {'surname': 'Abc', 'forename': 'Alice'},
                         'Wrong item was selected (%s)' % item)

    def testGetDictDepthThree(self):
        """get_item: testGetDictDepthThree"""
        path = 'person.name.forename'
        item = pythonpath.get_item(path, self.datadict)
        self.assertEqual(item, 'Alice',
                         'Wrong item was selected (%s)' % item)

    def testGetListDepthOne(self):
        """get_item: testGetListDepthOne"""
        path = '[1]'
        item = pythonpath.get_item(path, self.datalist)
        self.assertEqual(item, ['Bob', 'Bcd'],
                         'Wrong item was selected (%s)' % item)

    def testGetListDepthTwo(self):
        """get_item: testGetListDepthTwo"""
        path = '[0].[1]'
        item = pythonpath.get_item(path, self.datalist)
        self.assertEqual(item, 'Abc', 'Wrong item was selected (%s)' % item)


    def testGetMixedFirst(self):
        """get_item: testGetMixedFirst"""
        path = 'person[1].name'
        item = pythonpath.get_item(path, self.datamixed1)
        self.assertEqual(item, {'fore.name': 'Bob', 'surname': 'Bcd'},
                                'Wrong item was selected (%s)' % item)

    def testGetMixedSecond(self):
        """get_item: testGetMixedFirst"""
        path = 'person.name[0]'
        item = pythonpath.get_item(path, self.datamixed2)
        self.assertEqual(item, {'surname': 'Abc', 'forename': 'Alice'},
                                'Wrong item was selected (%s)' % item)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPythonPathParser))
    suite.addTest(unittest.makeSuite(TestPythonPathGetItem))
    return suite



if __name__ == '__main__':
    #unittest.main()
    unittest.TextTestRunner(verbosity=2).run(suite())

